@@ -49,6 +49,13 @@ __isl_give isl_val *isl_val_one(isl_ctx *ctx)
 	return isl_val_int_from_si(ctx, 1);
 }
 
+/* Return a reference to an isl_val representing negative one.
+ */
+__isl_give isl_val *isl_val_negone(isl_ctx *ctx)
+{
+	return isl_val_int_from_si(ctx, -1);
+}
+
 /* Return a reference to an isl_val representing NaN.
  */
 __isl_give isl_val *isl_val_nan(isl_ctx *ctx)
@@ -252,7 +259,7 @@ __isl_give isl_val *isl_val_cow(__isl_take isl_val *val)
 
 /* Free "v" and return NULL.
  */
-void *isl_val_free(__isl_take isl_val *v)
+__isl_null isl_val *isl_val_free(__isl_take isl_val *v)
 {
 	if (!v)
 		return NULL;
@@ -851,6 +858,17 @@ error:
 	return NULL;
 }
 
+/* Divide "v1" by "v2".
+ *
+ * This is a private copy of isl_val_div for use in the generic
+ * isl_multi_*_scale_down_val instantiated for isl_val.
+ */
+__isl_give isl_val *isl_val_scale_down_val(__isl_take isl_val *v1,
+	__isl_take isl_val *v2)
+{
+	return isl_val_div(v1, v2);
+}
+
 /* Given two integer values "v1" and "v2", check if "v1" is divisible by "v2".
  */
 int isl_val_is_divisible_by(__isl_keep isl_val *v1, __isl_keep isl_val *v2)
@@ -924,6 +942,48 @@ error:
 	return NULL;
 }
 
+/* Compute x, y and g such that g = gcd(a,b) and a*x+b*y = g.
+ */
+static void isl_int_gcdext(isl_int g, isl_int x, isl_int y,
+	isl_int a, isl_int b)
+{
+	isl_int d, tmp;
+	isl_int a_copy, b_copy;
+
+	isl_int_init(a_copy);
+	isl_int_init(b_copy);
+	isl_int_init(d);
+	isl_int_init(tmp);
+	isl_int_set(a_copy, a);
+	isl_int_set(b_copy, b);
+	isl_int_abs(g, a_copy);
+	isl_int_abs(d, b_copy);
+	isl_int_set_si(x, 1);
+	isl_int_set_si(y, 0);
+	while (isl_int_is_pos(d)) {
+		isl_int_fdiv_q(tmp, g, d);
+		isl_int_submul(x, tmp, y);
+		isl_int_submul(g, tmp, d);
+		isl_int_swap(g, d);
+		isl_int_swap(x, y);
+	}
+	if (isl_int_is_zero(a_copy))
+		isl_int_set_si(x, 0);
+	else if (isl_int_is_neg(a_copy))
+		isl_int_neg(x, x);
+	if (isl_int_is_zero(b_copy))
+		isl_int_set_si(y, 0);
+	else {
+		isl_int_mul(tmp, a_copy, x);
+		isl_int_sub(tmp, g, tmp);
+		isl_int_divexact(y, tmp, b_copy);
+	}
+	isl_int_clear(d);
+	isl_int_clear(tmp);
+	isl_int_clear(a_copy);
+	isl_int_clear(b_copy);
+}
+
 /* Given two integer values v1 and v2, return their greatest common divisor g,
  * as well as two integers x and y such that x * v1 + y * v2 = g.
  */
@@ -1281,6 +1341,33 @@ __isl_give isl_printer *isl_printer_print_val(__isl_take isl_printer *p,
 	return p;
 }
 
+/* Is "val1" (obviously) equal to "val2"?
+ *
+ * This is a private copy of isl_val_eq for use in the generic
+ * isl_multi_*_plain_is_equal instantiated for isl_val.
+ */
+int isl_val_plain_is_equal(__isl_keep isl_val *val1, __isl_keep isl_val *val2)
+{
+	return isl_val_eq(val1, val2);
+}
+
+/* Does "v" have any non-zero coefficients
+ * for any dimension in the given range?
+ *
+ * This function is only meant to be used in the generic isl_multi_*
+ * functions which have to deal with base objects that have an associated
+ * space.  Since an isl_val does not have any coefficients, this function
+ * always return 0.
+ */
+int isl_val_involves_dims(__isl_keep isl_val *v, enum isl_dim_type type,
+	unsigned first, unsigned n)
+{
+	if (!v)
+		return -1;
+
+	return 0;
+}
+
 /* Insert "n" dimensions of type "type" at position "first".
  *
  * This function is only meant to be used in the generic isl_multi_*
@@ -1320,6 +1407,22 @@ __isl_give isl_val *isl_val_set_dim_name(__isl_take isl_val *v,
 	return v;
 }
 
+/* Return the space of "v".
+ *
+ * This function is only meant to be used in the generic isl_multi_*
+ * functions which have to deal with base objects that have an associated
+ * space.  The conditions surrounding the call to this function make sure
+ * that this function will never actually get called.  We return a valid
+ * space anyway, just in case.
+ */
+__isl_give isl_space *isl_val_get_space(__isl_keep isl_val *v)
+{
+	if (!v)
+		return NULL;
+
+	return isl_space_params_alloc(isl_val_get_ctx(v), 0);
+}
+
 /* Reset the domain space of "v" to "space".
  *
  * This function is only meant to be used in the generic isl_multi_*
@@ -1336,6 +1439,24 @@ __isl_give isl_val *isl_val_reset_domain_space(__isl_take isl_val *v,
 	return v;
 }
 
+/* Align the parameters of "v" to those of "space".
+ *
+ * This function is only meant to be used in the generic isl_multi_*
+ * functions which have to deal with base objects that have an associated
+ * space.  Since an isl_val does not have an associated space, this function
+ * does not do anything, apart from error handling and cleaning up memory.
+ * Note that the conditions surrounding the call to this function make sure
+ * that this function will never actually get called.
+ */
+__isl_give isl_val *isl_val_align_params(__isl_take isl_val *v,
+	__isl_take isl_space *space)
+{
+	if (!space)
+		return isl_val_free(v);
+	isl_space_free(space);
+	return v;
+}
+
 /* Reorder the dimensions of the domain of "v" according
  * to the given reordering.
  *
@@ -1371,6 +1492,20 @@ __isl_give isl_val *isl_val_zero_on_domain(__isl_take isl_local_space *ls)
 	return isl_val_zero(ctx);
 }
 
+/* Do the parameters of "v" match those of "space"?
+ *
+ * This function is only meant to be used in the generic isl_multi_*
+ * functions which have to deal with base objects that have an associated
+ * space.  Since an isl_val does not have an associated space, this function
+ * simply returns 1, except if "v" or "space" are NULL.
+ */
+int isl_val_matching_params(__isl_keep isl_val *v, __isl_keep isl_space *space)
+{
+	if (!v || !space)
+		return -1;
+	return 1;
+}
+
 /* Check that the domain space of "v" matches "space".
  *
  * Return 0 on success and -1 on error.
@@ -1391,9 +1526,12 @@ int isl_val_check_match_domain_space(__isl_keep isl_val *v,
 #undef BASE
 #define BASE val
 
+#define NO_DOMAIN
+#define NO_INTERSECT_DOMAIN
 #define NO_GIST
 #define NO_IDENTITY
 #define NO_FROM_BASE
+#define NO_MOVE_DIMS
 #include <isl_multi_templ.c>
 
 /* Apply "fn" to each of the elements of "mv" with as second argument "v".
