@@ -56,6 +56,7 @@
 #ifdef OSL_SUPPORT
 #include <osl/scop.h>
 #include <osl/extensions/coordinates.h>
+#include <osl/extensions/loop.h>
 #endif
 
 /******************************************************************************
@@ -289,16 +290,19 @@ static void print_macros(FILE *file)
 	"#define ceild(n,d)  (((n)<0) ? -((-(n))/(d)) : ((n)+(d)-1)/(d))\n");
     fprintf(file, "#define max(x,y)    ((x) > (y) ? (x) : (y))\n") ; 
     fprintf(file, "#define min(x,y)    ((x) < (y) ? (x) : (y))\n\n") ; 
+    fprintf(file, "#ifdef TIME \n#define IF_TIME(foo) foo; \n"
+                  "#else\n#define IF_TIME(foo)\n#endif\n\n");
 }
 
-static void print_declarations(FILE *file, int n, char **names)
+static void print_declarations(FILE *file, int n, char **names, int indentation)
 {
     int i;
 
-    fprintf(file, "  int %s", names[0]); 
+    for (i = 0; i < indentation; i++)
+      fprintf(file, " ");
+    fprintf(file, "int %s", names[0]);
     for (i = 1; i < n; i++)
-	fprintf(file, ", %s", names[i]); 
-
+	fprintf(file, ", %s", names[i]);
     fprintf(file, ";\n");
 }
 
@@ -309,11 +313,11 @@ static void print_iterator_declarations(FILE *file, CloogProgram *program,
 
     if (names->nb_scattering) {
 	fprintf(file, "  /* Scattering iterators. */\n");
-	print_declarations(file, names->nb_scattering, names->scattering);
+	print_declarations(file, names->nb_scattering, names->scattering, 2);
     }
     if (names->nb_iterators) {
 	fprintf(file, "  /* Original iterators. */\n");
-	print_declarations(file, names->nb_iterators, names->iterators);
+	print_declarations(file, names->nb_iterators, names->iterators, 2);
     }
 }
 
@@ -359,6 +363,112 @@ static void print_callable_postamble(FILE *file, CloogProgram *program)
     fprintf(file, "}\n"); 
 }
 
+#ifdef OSL_SUPPORT
+static int get_osl_loop_flags (osl_scop_p scop) {
+  int flags = 0;
+  osl_loop_p ll = osl_generic_lookup(scop->extension, OSL_URI_LOOP);
+  while (ll) {
+    flags |= ll->directive;
+    ll = ll->next;
+  }
+
+  return flags;
+}
+
+static void print_iterator_declarations_osl(FILE *file, CloogProgram *program,
+    int indent, CloogOptions *options)
+{
+  osl_coordinates_p co = NULL;
+  int i;
+  int loopflags = 0;
+  char* vecvar[2] = {"lbv", "ubv"};
+  char* parvar[2] = {"lbp", "ubp"};
+
+  osl_scop_p scop = options->scop;
+  CloogNames *names = program->names;
+
+  if (names->nb_scattering) {
+    for (i = 0; i < indent; i++)
+      fprintf(file, " ");
+    fprintf(file, "/* Scattering iterators. */\n");
+    print_declarations(file, names->nb_scattering, names->scattering, indent);
+  }
+
+  co = osl_generic_lookup(scop->extension, OSL_URI_COORDINATES);
+  if (co==NULL //if coordinates exist then iterators already declared in file
+      && names->nb_iterators) {
+    for (i = 0; i < indent; i++)
+      fprintf(file, " ");
+    fprintf(file, "/* Original iterators. */\n");
+    print_declarations(file, names->nb_iterators, names->iterators, indent);
+  }
+
+  loopflags = get_osl_loop_flags(scop);
+  if(loopflags & CLAST_PARALLEL_OMP)
+    print_declarations(file, 2, parvar, indent);
+  if(loopflags & CLAST_PARALLEL_VEC)
+    print_declarations(file, 2, vecvar, indent);
+  
+  fprintf(file, "\n");
+}
+
+/*
+* add tags clast loops according to information in scop's osl_loop extension
+*/
+static int annotate_loops(osl_scop_p program, struct clast_stmt *root){
+
+  int j, nclastloops, nclaststmts;
+  struct clast_for **clastloops = NULL;
+  int *claststmts = NULL;
+  int ret = 0;
+
+  if (program == NULL) {
+    return ret;
+  }
+
+  osl_loop_p ll = osl_generic_lookup(program->extension, OSL_URI_LOOP);
+  while (ll) {
+    //for each loop
+    osl_loop_p loop = ll;
+    ClastFilter filter = { loop->iter, loop->stmt_ids, 
+                           loop->nb_stmts, subset};
+
+    clast_filter(root, filter, &clastloops, &nclastloops, 
+                 &claststmts, &nclaststmts);
+
+    /* There should be at least one */
+    if (nclastloops==0) {  //FROM PLUTO
+       /* Sometimes loops may disappear (1) tile size larger than trip count
+       * 2) it's a scalar dimension but can't be determined from the
+       * trans matrix */
+       printf("Warning: parallel poly loop not found in AST\n");
+       ll = ll->next;
+       continue;
+    }
+    for (j=0; j<nclastloops; j++) {
+
+      if (loop->directive & CLAST_PARALLEL_VEC) {
+        clastloops[j]->parallel |= CLAST_PARALLEL_VEC;
+        ret |= CLAST_PARALLEL_VEC;
+      }
+
+      if (loop->directive & CLAST_PARALLEL_OMP) {
+        clastloops[j]->parallel |= CLAST_PARALLEL_OMP;
+        ret |= CLAST_PARALLEL_OMP;
+        clastloops[j]->private_vars = strdup(loop->private_vars);
+      }
+    }
+
+    if (clastloops) { free(clastloops); clastloops=NULL;}
+    if (claststmts) { free(claststmts); claststmts=NULL;}
+
+    ll = ll->next;
+  }
+
+  return ret;
+}
+#endif
+
 /**
  * cloog_program_osl_pprint function:
  * this function pretty-prints the C or FORTRAN code generated from an
@@ -376,56 +486,86 @@ int cloog_program_osl_pprint(FILE * file, CloogProgram * program,
                              CloogOptions * options) {
 #ifdef OSL_SUPPORT
   int lines = 0;
+  int columns = 0;
   int read = 1;
+  int indentation = 0;
   char c;
   osl_scop_p scop = options->scop;
   osl_coordinates_p coordinates;
   struct clast_stmt *root;
-  FILE * original;
+  FILE* original = NULL;
 
   if (scop && !options->compilable && !options->callable) {
+#ifdef CLOOG_RUSAGE
+    print_comment(file, options, "Generated from %s by %s in %.2fs.",
+                  options->name, cloog_version(), options->time);
+#else
+    print_comment(file, options, "Generated from %s by %s.",
+                  options->name, cloog_version());
+#endif
     coordinates = osl_generic_lookup(scop->extension, OSL_URI_COORDINATES);
     if (coordinates) {
       original = fopen(coordinates->name, "r");
+      indentation = coordinates->indent;
       if (!original) {
         cloog_msg(options, CLOOG_WARNING,
                   "unable to open the file specified in the SCoP "
                   "coordinates\n");
-        return 0;
+        coordinates = NULL;
       }
+    }
 
-      /* Print the macros the generated code may need. */
-      print_macros(file);
+    /* Print the macros the generated code may need. */
+    print_macros(file);
 
-      /* Print what was before the SCoP in the original file. */
-      while ((lines < coordinates->start) && (read != EOF)) {
+    /* Print what was before the SCoP in the original file (if any). */
+    if (coordinates) {
+      while (((lines < coordinates->line_start - 1) ||
+              (columns < coordinates->column_start - 1)) && (read != EOF)) {
         read = fscanf(original, "%c", &c);
+        columns++;
         if (read != EOF) {
-          if (c == '\n')
-            lines ++;
+          if (c == '\n') {
+            lines++;
+            columns = 0;
+          }
           fprintf(file, "%c", c);
         }
       }
 
-      /* Generate the clast from the pseudo-AST then pretty-print it. */
-      root = cloog_clast_create(program, options);
-      clast_pprint(file, root, coordinates->indent, options);
-      cloog_clast_free(root);
+      /* Carriage return to preserve indentation if necessary. */
+      if (coordinates->column_start > 0)
+        fprintf(file, "\n");
+    }
 
-      /* Print what was after the SCoP in the original file. */
+    /* Generate the clast from the pseudo-AST then pretty-print it. */
+    root = cloog_clast_create(program, options);
+    annotate_loops(options->scop, root);
+    print_iterator_declarations_osl(file, program, indentation, options);
+    clast_pprint(file, root, indentation, options);
+    cloog_clast_free(root);
+
+    /* Print what was after the SCoP in the original file (if any). */
+    if (coordinates) {
       while (read != EOF) {
         read = fscanf(original, "%c", &c);
+        columns++;
         if (read != EOF) {
-          if (lines >= coordinates->end - 1)
+          if (((lines == coordinates->line_end - 1) &&
+               (columns > coordinates->column_end)) ||
+              (lines > coordinates->line_end - 1))
             fprintf(file, "%c", c);
-          if (c == '\n')
-            lines ++;
+          if (c == '\n') {
+            lines++;
+            columns = 0;
+          }
         }
       }
 
       fclose(original);
-      return 1;
     }
+
+    return 1;
   }
 #endif
   return 0;
